// Copyright (c) Sandeep Mistry. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.

#include "BLETrackBall.h"

static const uint8_t descriptorValue[] PROGMEM = {
  /* Trackball with 3 axis and 2 button*/
  0x05, 0x01,             /* USAGE_PAGE (Generic Desktop) */
  0x09, 0x04,             /* USAGE (Joystick) */
  0xa1, 0x01,             /* COLLECTION (Application) */
  
  /* 3 16bit Axis */
  0x05, 0x01,             /*   USAGE_PAGE (Generic Desktop) */
  0xa1, 0x00,             /*   COLLECTION (Physical) */
  0x09, 0x30,             /*     USAGE (X) */
  0x09, 0x31,             /*     USAGE (Y) */
  0x09, 0x32,             /*     USAGE (X) */
  0x09, 0x33,             /*     USAGE (Rx) */
  0x09, 0x34,             /*     USAGE (Ry) */
  0x09, 0x35,             /*     USAGE (Rz) */
  0x15, 0x80,             /*     LOGICAL_MINIMUM (-128) */
  0x25, 0x7F,             /*     LOGICAL_MAXIMUM (127) */  
  0x75, 0x08,             /*     REPORT_SIZE (8) */
  0x95, 0x06,             /*     REPORT_COUNT (6) */
  0x81, 0x02,             /*     INPUT (Data,Var,Abs) */
  0xc0,               /*   END_COLLECTION */
  0xc0                /* END_COLLECTION */
};

//0x16, 0x00, 0x80,         /*     LOGICAL_MINIMUM (-32768) */
//0x26, 0xFF, 0x7F,         /*     LOGICAL_MAXIMUM (32767) */

BLETrackBall::BLETrackBall() :
  BLEHID(descriptorValue, sizeof(descriptorValue), 11),
  _reportCharacteristic("2a4d", BLERead | BLENotify, 6),
  _reportReferenceDescriptor(BLEHIDDescriptorTypeInput),
  _button(0)
{
}

void BLETrackBall::move(int8_t x, int8_t y, int8_t z, int8_t rx, int8_t ry, int8_t rz) {
  unsigned char mouseMove[6]= { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };

  // send key code
  mouseMove[0] = x;
  mouseMove[1] = y;
  mouseMove[2] = z;
  mouseMove[3] = rx;
  mouseMove[4] = ry;
  mouseMove[5] = rz;

  this->sendData(this->_reportCharacteristic, mouseMove, sizeof(mouseMove));
}


void BLETrackBall::setReportId(unsigned char reportId) {
  BLEHID::setReportId(reportId);

  this->_reportReferenceDescriptor.setReportId(reportId);
}

unsigned char BLETrackBall::numAttributes() {
  return 2;
}

BLELocalAttribute** BLETrackBall::attributes() {
  static BLELocalAttribute* attributes[2];

  attributes[0] = &this->_reportCharacteristic;
  attributes[1] = &this->_reportReferenceDescriptor;

  return attributes;
}
